﻿package
{
    import flash.display.Sprite;
    import flash.display.StageAlign;
    import flash.display.StageScaleMode;
    import flash.events.AccelerometerEvent;
    import flash.events.Event; 
    import flash.events.EventDispatcher;
    import flash.sensors.Accelerometer;
    import flash.text.TextField;
    import flash.text.TextFormat;    

    /**
     * Document class for Shakey application. 
     * Android Development with Flash sample app 
	 * 
	 * Shake logic credit from Objective-C example:  
	 * http://stackoverflow.com/questions/150446/how-do-i-detect-when-someone-shakes-an-iphone#159610
     *  
     * @author Rich Wagner
     * 
     */            
    [Event("shake", "flash.events.Event")]    
    public class Shakey extends Sprite 
    {
        // Class properties  
        private var accelerometer:Accelerometer;          
        private var tfInfo:TextField;         
        
        // Data used to determine a shake 
        private var deltaX:Number; 
        private var deltaY:Number; 
        private var deltaZ:Number; 
        private var accX:Number; 
        private var accY:Number; 
        private var accZ:Number;   
        
        // Number of shakes captured 
        private var shakes:int = 0; 
        
        // Constant used to determine whether a movement is strong enough 
        // to constitute a "shake" 
        private var threshold:Number  = 0.7; 
        
        /**
         * Constructor  
         * 
         */
        public function Shakey()
        {
            stage.scaleMode = StageScaleMode.NO_SCALE;
            stage.align = StageAlign.TOP_LEFT;

            // Create text field 
            tfInfo = new TextField();
            var format:TextFormat = new TextFormat("Helvetica", 26);
            format.color = 0x000000; 
            tfInfo.defaultTextFormat = format;
            tfInfo.border = false;
            tfInfo.wordWrap = true;
            tfInfo.multiline = true;
            tfInfo.x = 10;
            tfInfo.y = 10;
            tfInfo.height = stage.stageHeight - 80;
            tfInfo.width = stage.stageWidth - 20;            
            addChild(tfInfo);                        
            
            // Determine accelerator support 
            if (Accelerometer.isSupported)
            {
                accelerometer = new Accelerometer();
                accelerometer.addEventListener(AccelerometerEvent.UPDATE, accelerometerUpdateHandler);
            }        
            else
            {
                tfInfo.text = "Motion sensor detection is not support on this device. No shaking for you!"; 
            }    

            // Add custom shake event 
            addEventListener("shake", shakeHandler); 
            
        }
        

        /**
         * Handler for Accelerator updates  
         * 
         * @param event
         * 
         */
        private function accelerometerUpdateHandler(event:AccelerometerEvent):void 
        {
            
            // Determine delta for each dimension 
            deltaX = Math.abs(accX - event.accelerationX);
            deltaY = Math.abs(accY - event.accelerationY);
            deltaZ = Math.abs(accZ - event.accelerationZ);
            
            // Algorthym to determine whether a shake occurred.  
            if ((deltaX > threshold && deltaY > threshold) ||
                (deltaX > threshold && deltaZ > threshold) ||
                (deltaY > threshold && deltaZ > threshold))                
            {                
                //tfInfo.text = "dispatchShake"; 
                dispatchEvent(new Event("shake")); 
            }
            
            // Save acceleration data from this read 
            accX = event.accelerationX;
            accY = event.accelerationY;
            accZ = event.accelerationZ;
            
            //tfInfo.text = "x: " + accX + " y: " + accY + " z: " + accZ;
        }       

        /**
         * Handler for shake event 
         *  
         * @param event
         * 
         */
        private function shakeHandler(event:Event):void
        {
            shakes++; 
            tfInfo.text = "Shakes detected: " + shakes.toString();  
        }
                
        
        
    }
}